/* remote-machine.c generated by valac 0.17.5.2-8558, the Vala compiler
 * generated from remote-machine.vala, do not modify */

/* This file is part of GNOME Boxes. License: LGPLv2+*/

#include <glib.h>
#include <glib-object.h>
#include <clutter/clutter.h>
#include <gio/gio.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <libcommon.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>


#define BOXES_TYPE_UI (boxes_ui_get_type ())
#define BOXES_UI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_UI, BoxesUI))
#define BOXES_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_UI, BoxesUIClass))
#define BOXES_IS_UI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_UI))
#define BOXES_IS_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_UI))
#define BOXES_UI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_UI, BoxesUIClass))

typedef struct _BoxesUI BoxesUI;
typedef struct _BoxesUIClass BoxesUIClass;
typedef struct _BoxesUIPrivate BoxesUIPrivate;

#define BOXES_TYPE_UI_STATE (boxes_ui_state_get_type ())

#define BOXES_TYPE_COLLECTION_ITEM (boxes_collection_item_get_type ())
#define BOXES_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItem))
#define BOXES_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))
#define BOXES_IS_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_IS_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_COLLECTION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))

typedef struct _BoxesCollectionItem BoxesCollectionItem;
typedef struct _BoxesCollectionItemClass BoxesCollectionItemClass;
typedef struct _BoxesCollectionItemPrivate BoxesCollectionItemPrivate;

#define BOXES_TYPE_IPROPERTIES_PROVIDER (boxes_iproperties_provider_get_type ())
#define BOXES_IPROPERTIES_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER, BoxesIPropertiesProvider))
#define BOXES_IS_IPROPERTIES_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER))
#define BOXES_IPROPERTIES_PROVIDER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER, BoxesIPropertiesProviderIface))

typedef struct _BoxesIPropertiesProvider BoxesIPropertiesProvider;
typedef struct _BoxesIPropertiesProviderIface BoxesIPropertiesProviderIface;

#define BOXES_TYPE_PROPERTIES_PAGE (boxes_properties_page_get_type ())

#define BOXES_TYPE_PROPERTY (boxes_property_get_type ())
#define BOXES_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_PROPERTY, BoxesProperty))
#define BOXES_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_PROPERTY, BoxesPropertyClass))
#define BOXES_IS_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_PROPERTY))
#define BOXES_IS_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_PROPERTY))
#define BOXES_PROPERTY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_PROPERTY, BoxesPropertyClass))

typedef struct _BoxesProperty BoxesProperty;
typedef struct _BoxesPropertyClass BoxesPropertyClass;

#define BOXES_TYPE_MACHINE (boxes_machine_get_type ())
#define BOXES_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_MACHINE, BoxesMachine))
#define BOXES_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_MACHINE, BoxesMachineClass))
#define BOXES_IS_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_MACHINE))
#define BOXES_IS_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_MACHINE))
#define BOXES_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_MACHINE, BoxesMachineClass))

typedef struct _BoxesMachine BoxesMachine;
typedef struct _BoxesMachineClass BoxesMachineClass;
typedef struct _BoxesMachinePrivate BoxesMachinePrivate;

#define BOXES_TYPE_MACHINE_ACTOR (boxes_machine_actor_get_type ())
#define BOXES_MACHINE_ACTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_MACHINE_ACTOR, BoxesMachineActor))
#define BOXES_MACHINE_ACTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_MACHINE_ACTOR, BoxesMachineActorClass))
#define BOXES_IS_MACHINE_ACTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_MACHINE_ACTOR))
#define BOXES_IS_MACHINE_ACTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_MACHINE_ACTOR))
#define BOXES_MACHINE_ACTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_MACHINE_ACTOR, BoxesMachineActorClass))

typedef struct _BoxesMachineActor BoxesMachineActor;
typedef struct _BoxesMachineActorClass BoxesMachineActorClass;

#define BOXES_TYPE_REMOTE_MACHINE (boxes_remote_machine_get_type ())
#define BOXES_REMOTE_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_REMOTE_MACHINE, BoxesRemoteMachine))
#define BOXES_REMOTE_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_REMOTE_MACHINE, BoxesRemoteMachineClass))
#define BOXES_IS_REMOTE_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_REMOTE_MACHINE))
#define BOXES_IS_REMOTE_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_REMOTE_MACHINE))
#define BOXES_REMOTE_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_REMOTE_MACHINE, BoxesRemoteMachineClass))

typedef struct _BoxesRemoteMachine BoxesRemoteMachine;
typedef struct _BoxesRemoteMachineClass BoxesRemoteMachineClass;
typedef struct _BoxesRemoteMachinePrivate BoxesRemoteMachinePrivate;
#define _g_free0(var) (var = (g_free (var), NULL))

#define BOXES_MACHINE_TYPE_MACHINE_STATE (boxes_machine_machine_state_get_type ())
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define BOXES_TYPE_DISPLAY (boxes_display_get_type ())
#define BOXES_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_DISPLAY, BoxesDisplay))
#define BOXES_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_DISPLAY, BoxesDisplayClass))
#define BOXES_IS_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_DISPLAY))
#define BOXES_IS_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_DISPLAY))
#define BOXES_DISPLAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_DISPLAY, BoxesDisplayClass))

typedef struct _BoxesDisplay BoxesDisplay;
typedef struct _BoxesDisplayClass BoxesDisplayClass;

#define BOXES_TYPE_SPICE_DISPLAY (boxes_spice_display_get_type ())
#define BOXES_SPICE_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SPICE_DISPLAY, BoxesSpiceDisplay))
#define BOXES_SPICE_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SPICE_DISPLAY, BoxesSpiceDisplayClass))
#define BOXES_IS_SPICE_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SPICE_DISPLAY))
#define BOXES_IS_SPICE_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SPICE_DISPLAY))
#define BOXES_SPICE_DISPLAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SPICE_DISPLAY, BoxesSpiceDisplayClass))

typedef struct _BoxesSpiceDisplay BoxesSpiceDisplay;
typedef struct _BoxesSpiceDisplayClass BoxesSpiceDisplayClass;

#define BOXES_TYPE_VNC_DISPLAY (boxes_vnc_display_get_type ())
#define BOXES_VNC_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_VNC_DISPLAY, BoxesVncDisplay))
#define BOXES_VNC_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_VNC_DISPLAY, BoxesVncDisplayClass))
#define BOXES_IS_VNC_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_VNC_DISPLAY))
#define BOXES_IS_VNC_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_VNC_DISPLAY))
#define BOXES_VNC_DISPLAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_VNC_DISPLAY, BoxesVncDisplayClass))

typedef struct _BoxesVncDisplay BoxesVncDisplay;
typedef struct _BoxesVncDisplayClass BoxesVncDisplayClass;
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
typedef struct _BoxesRemoteMachineConnectDisplayData BoxesRemoteMachineConnectDisplayData;
#define __g_list_free__g_object_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_object_unref0_ (var), NULL)))

typedef enum  {
	BOXES_UI_STATE_NONE,
	BOXES_UI_STATE_COLLECTION,
	BOXES_UI_STATE_CREDS,
	BOXES_UI_STATE_DISPLAY,
	BOXES_UI_STATE_SETTINGS,
	BOXES_UI_STATE_WIZARD,
	BOXES_UI_STATE_PROPERTIES
} BoxesUIState;

struct _BoxesUI {
	GObject parent_instance;
	BoxesUIPrivate * priv;
	BoxesUIState previous_ui_state;
};

struct _BoxesUIClass {
	GObjectClass parent_class;
	void (*ui_state_changed) (BoxesUI* self);
	ClutterActor* (*get_actor) (BoxesUI* self);
};

struct _BoxesCollectionItem {
	BoxesUI parent_instance;
	BoxesCollectionItemPrivate * priv;
};

struct _BoxesCollectionItemClass {
	BoxesUIClass parent_class;
};

typedef enum  {
	BOXES_PROPERTIES_PAGE_LOGIN,
	BOXES_PROPERTIES_PAGE_SYSTEM,
	BOXES_PROPERTIES_PAGE_DISPLAY,
	BOXES_PROPERTIES_PAGE_DEVICES,
	BOXES_PROPERTIES_PAGE_LAST
} BoxesPropertiesPage;

struct _BoxesIPropertiesProviderIface {
	GTypeInterface parent_iface;
	GList* (*get_properties) (BoxesIPropertiesProvider* self, BoxesPropertiesPage page);
};

struct _BoxesMachine {
	BoxesCollectionItem parent_instance;
	BoxesMachinePrivate * priv;
	BoxesMachineActor* machine_actor;
	BoxesCollectionSource* source;
	BoxesDisplayConfig* config;
	gboolean stay_on_display;
	gboolean deleted;
};

struct _BoxesMachineClass {
	BoxesCollectionItemClass parent_class;
	void (*take_screenshot) (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GdkPixbuf* (*take_screenshot_finish) (BoxesMachine* self, GAsyncResult* _res_, GError** error);
	GList* (*get_properties) (BoxesMachine* self, BoxesPropertiesPage page);
	void (*connect_display) (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*connect_display_finish) (BoxesMachine* self, GAsyncResult* _res_);
	void (*disconnect_display) (BoxesMachine* self);
	void (*delete) (BoxesMachine* self, gboolean by_user);
};

struct _BoxesRemoteMachine {
	BoxesMachine parent_instance;
	BoxesRemoteMachinePrivate * priv;
};

struct _BoxesRemoteMachineClass {
	BoxesMachineClass parent_class;
};

typedef enum  {
	BOXES_MACHINE_MACHINE_STATE_UNKNOWN,
	BOXES_MACHINE_MACHINE_STATE_STOPPED,
	BOXES_MACHINE_MACHINE_STATE_RUNNING,
	BOXES_MACHINE_MACHINE_STATE_PAUSED,
	BOXES_MACHINE_MACHINE_STATE_SAVED
} BoxesMachineMachineState;

struct _BoxesRemoteMachineConnectDisplayData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesRemoteMachine* self;
	BoxesDisplay* _tmp0_;
	BoxesDisplay* _tmp1_;
	BoxesCollectionSource* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gboolean _tmp6_;
	BoxesDisplayConfig* _tmp7_;
	BoxesCollectionSource* _tmp8_;
	gchar* _tmp9_;
	gchar* _tmp10_;
	gchar* _tmp11_;
	BoxesSpiceDisplay* _tmp12_;
	BoxesSpiceDisplay* _tmp13_;
	BoxesCollectionSource* _tmp14_;
	gchar* _tmp15_;
	gchar* _tmp16_;
	gchar* _tmp17_;
	gboolean _tmp18_;
	BoxesDisplayConfig* _tmp19_;
	BoxesCollectionSource* _tmp20_;
	gchar* _tmp21_;
	gchar* _tmp22_;
	gchar* _tmp23_;
	BoxesVncDisplay* _tmp24_;
	BoxesVncDisplay* _tmp25_;
	BoxesVncDisplay* _tmp26_;
	BoxesVncDisplay* _tmp27_;
	BoxesDisplay* _tmp28_;
	BoxesDisplay* _tmp29_;
	GError* _error_;
	GError* _tmp30_;
	const gchar* _tmp31_;
	GError * _inner_error_;
};

typedef void (*PropertyStringChanged) (BoxesProperty* property, const gchar* value, void* user_data, GError** error);

static gpointer boxes_remote_machine_parent_class = NULL;
static BoxesIPropertiesProviderIface* boxes_remote_machine_boxes_iproperties_provider_parent_iface = NULL;

GType boxes_ui_get_type (void) G_GNUC_CONST;
GType boxes_ui_state_get_type (void) G_GNUC_CONST;
GType boxes_collection_item_get_type (void) G_GNUC_CONST;
GType boxes_properties_page_get_type (void) G_GNUC_CONST;
GType boxes_property_get_type (void) G_GNUC_CONST;
GType boxes_iproperties_provider_get_type (void) G_GNUC_CONST;
GType boxes_machine_get_type (void) G_GNUC_CONST;
GType boxes_machine_actor_get_type (void) G_GNUC_CONST;
GType boxes_remote_machine_get_type (void) G_GNUC_CONST;
enum  {
	BOXES_REMOTE_MACHINE_DUMMY_PROPERTY
};
BoxesRemoteMachine* boxes_remote_machine_new (BoxesCollectionSource* source);
BoxesRemoteMachine* boxes_remote_machine_construct (GType object_type, BoxesCollectionSource* source);
BoxesMachine* boxes_machine_construct (GType object_type, BoxesCollectionSource* source, const gchar* name);
GType boxes_machine_machine_state_get_type (void) G_GNUC_CONST;
void boxes_machine_set_state (BoxesMachine* self, BoxesMachineMachineState value);
void boxes_machine_create_display_config (BoxesMachine* self, const gchar* uuid);
void boxes_machine_load_screenshot (BoxesMachine* self);
static void boxes_remote_machine_real_connect_display_data_free (gpointer _data);
static void boxes_remote_machine_real_connect_display (BoxesMachine* base, GAsyncReadyCallback _callback_, gpointer _user_data_);
static gboolean boxes_remote_machine_real_connect_display_co (BoxesRemoteMachineConnectDisplayData* _data_);
GType boxes_display_get_type (void) G_GNUC_CONST;
BoxesDisplay* boxes_machine_get_display (BoxesMachine* self);
BoxesSpiceDisplay* boxes_spice_display_new_with_uri (BoxesDisplayConfig* config, const gchar* uri);
BoxesSpiceDisplay* boxes_spice_display_construct_with_uri (GType object_type, BoxesDisplayConfig* config, const gchar* uri);
GType boxes_spice_display_get_type (void) G_GNUC_CONST;
void boxes_machine_set_display (BoxesMachine* self, BoxesDisplay* value);
BoxesVncDisplay* boxes_vnc_display_new_with_uri (BoxesDisplayConfig* config, const gchar* _uri, GError** error);
BoxesVncDisplay* boxes_vnc_display_construct_with_uri (GType object_type, BoxesDisplayConfig* config, const gchar* _uri, GError** error);
GType boxes_vnc_display_get_type (void) G_GNUC_CONST;
void boxes_display_connect_it (BoxesDisplay* self);
static GList* boxes_remote_machine_real_get_properties (BoxesIPropertiesProvider* base, BoxesPropertiesPage page);
BoxesProperty* boxes_iproperties_provider_add_string_property (BoxesIPropertiesProvider* self, GList** list, const gchar* name, const gchar* value, PropertyStringChanged changed, void* changed_target);
static void ___lambda129_ (BoxesRemoteMachine* self, BoxesProperty* property, const gchar* name, GError** error);
static void ____lambda129__property_string_changed (BoxesProperty* property, const gchar* value, gpointer self, GError** error);
GList* boxes_display_get_properties (BoxesDisplay* self, BoxesPropertiesPage page);
static void _g_object_unref0_ (gpointer var);
static void _g_list_free__g_object_unref0_ (GList* self);
static void boxes_remote_machine_real_delete (BoxesMachine* base, gboolean by_user);


BoxesRemoteMachine* boxes_remote_machine_construct (GType object_type, BoxesCollectionSource* source) {
	BoxesRemoteMachine * self = NULL;
	BoxesCollectionSource* _tmp0_;
	BoxesCollectionSource* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	BoxesCollectionSource* _tmp5_;
	g_return_val_if_fail (source != NULL, NULL);
	_tmp0_ = source;
	_tmp1_ = source;
	_tmp2_ = boxes_collection_source_get_name (_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp4_ = _tmp3_;
	self = (BoxesRemoteMachine*) boxes_machine_construct (object_type, _tmp0_, _tmp4_);
	_g_free0 (_tmp4_);
	boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_RUNNING);
	boxes_machine_create_display_config ((BoxesMachine*) self, NULL);
	_tmp5_ = source;
	g_object_bind_property_with_closures ((GObject*) _tmp5_, "name", (GObject*) self, "name", G_BINDING_DEFAULT, (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)), (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)));
	boxes_machine_load_screenshot ((BoxesMachine*) self);
	return self;
}


BoxesRemoteMachine* boxes_remote_machine_new (BoxesCollectionSource* source) {
	return boxes_remote_machine_construct (BOXES_TYPE_REMOTE_MACHINE, source);
}


static void boxes_remote_machine_real_connect_display_data_free (gpointer _data) {
	BoxesRemoteMachineConnectDisplayData* _data_;
	_data_ = _data;
	_g_object_unref0 (_data_->self);
	g_slice_free (BoxesRemoteMachineConnectDisplayData, _data_);
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static void boxes_remote_machine_real_connect_display (BoxesMachine* base, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesRemoteMachine * self;
	BoxesRemoteMachineConnectDisplayData* _data_;
	BoxesRemoteMachine* _tmp0_;
	self = (BoxesRemoteMachine*) base;
	_data_ = g_slice_new0 (BoxesRemoteMachineConnectDisplayData);
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_remote_machine_real_connect_display);
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_remote_machine_real_connect_display_data_free);
	_tmp0_ = _g_object_ref0 (self);
	_data_->self = _tmp0_;
	boxes_remote_machine_real_connect_display_co (_data_);
}


static void boxes_remote_machine_real_connect_display_finish (BoxesMachine* base, GAsyncResult* _res_) {
	BoxesRemoteMachineConnectDisplayData* _data_;
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
}


static gboolean boxes_remote_machine_real_connect_display_co (BoxesRemoteMachineConnectDisplayData* _data_) {
	switch (_data_->_state_) {
		case 0:
		goto _state_0;
		default:
		g_assert_not_reached ();
	}
	_state_0:
	_data_->_tmp0_ = boxes_machine_get_display ((BoxesMachine*) _data_->self);
	_data_->_tmp1_ = _data_->_tmp0_;
	if (_data_->_tmp1_ != NULL) {
		if (_data_->_state_ == 0) {
			g_simple_async_result_complete_in_idle (_data_->_async_result);
		} else {
			g_simple_async_result_complete (_data_->_async_result);
		}
		g_object_unref (_data_->_async_result);
		return FALSE;
	}
	{
		_data_->_tmp2_ = ((BoxesMachine*) _data_->self)->source;
		_data_->_tmp3_ = boxes_collection_source_get_source_type (_data_->_tmp2_);
		_data_->_tmp4_ = _data_->_tmp3_;
		_data_->_tmp5_ = _data_->_tmp4_;
		_data_->_tmp6_ = g_strcmp0 (_data_->_tmp5_, "spice") == 0;
		_g_free0 (_data_->_tmp5_);
		if (_data_->_tmp6_) {
			_data_->_tmp7_ = ((BoxesMachine*) _data_->self)->config;
			_data_->_tmp8_ = ((BoxesMachine*) _data_->self)->source;
			_data_->_tmp9_ = boxes_collection_source_get_uri (_data_->_tmp8_);
			_data_->_tmp10_ = _data_->_tmp9_;
			_data_->_tmp11_ = _data_->_tmp10_;
			_data_->_tmp12_ = boxes_spice_display_new_with_uri (_data_->_tmp7_, _data_->_tmp11_);
			_data_->_tmp13_ = _data_->_tmp12_;
			boxes_machine_set_display ((BoxesMachine*) _data_->self, (BoxesDisplay*) _data_->_tmp13_);
			_g_object_unref0 (_data_->_tmp13_);
			_g_free0 (_data_->_tmp11_);
		} else {
			_data_->_tmp14_ = ((BoxesMachine*) _data_->self)->source;
			_data_->_tmp15_ = boxes_collection_source_get_source_type (_data_->_tmp14_);
			_data_->_tmp16_ = _data_->_tmp15_;
			_data_->_tmp17_ = _data_->_tmp16_;
			_data_->_tmp18_ = g_strcmp0 (_data_->_tmp17_, "vnc") == 0;
			_g_free0 (_data_->_tmp17_);
			if (_data_->_tmp18_) {
				_data_->_tmp19_ = ((BoxesMachine*) _data_->self)->config;
				_data_->_tmp20_ = ((BoxesMachine*) _data_->self)->source;
				_data_->_tmp21_ = boxes_collection_source_get_uri (_data_->_tmp20_);
				_data_->_tmp22_ = _data_->_tmp21_;
				_data_->_tmp23_ = _data_->_tmp22_;
				_data_->_tmp24_ = boxes_vnc_display_new_with_uri (_data_->_tmp19_, _data_->_tmp23_, &_data_->_inner_error_);
				_data_->_tmp25_ = _data_->_tmp24_;
				_g_free0 (_data_->_tmp23_);
				_data_->_tmp26_ = _data_->_tmp25_;
				if (_data_->_inner_error_ != NULL) {
					goto __catch47_g_error;
				}
				_data_->_tmp27_ = _data_->_tmp26_;
				boxes_machine_set_display ((BoxesMachine*) _data_->self, (BoxesDisplay*) _data_->_tmp27_);
				_g_object_unref0 (_data_->_tmp27_);
			}
		}
		_data_->_tmp28_ = boxes_machine_get_display ((BoxesMachine*) _data_->self);
		_data_->_tmp29_ = _data_->_tmp28_;
		boxes_display_connect_it (_data_->_tmp29_);
	}
	goto __finally47;
	__catch47_g_error:
	{
		_data_->_error_ = _data_->_inner_error_;
		_data_->_inner_error_ = NULL;
		_data_->_tmp30_ = _data_->_error_;
		_data_->_tmp31_ = _data_->_tmp30_->message;
		g_warning ("remote-machine.vala:30: %s", _data_->_tmp31_);
		_g_error_free0 (_data_->_error_);
	}
	__finally47:
	if (_data_->_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
		g_clear_error (&_data_->_inner_error_);
		return FALSE;
	}
	if (_data_->_state_ == 0) {
		g_simple_async_result_complete_in_idle (_data_->_async_result);
	} else {
		g_simple_async_result_complete (_data_->_async_result);
	}
	g_object_unref (_data_->_async_result);
	return FALSE;
}


static void ___lambda129_ (BoxesRemoteMachine* self, BoxesProperty* property, const gchar* name, GError** error) {
	BoxesCollectionSource* _tmp0_;
	const gchar* _tmp1_;
	g_return_if_fail (property != NULL);
	g_return_if_fail (name != NULL);
	_tmp0_ = ((BoxesMachine*) self)->source;
	_tmp1_ = name;
	boxes_collection_source_set_name (_tmp0_, _tmp1_);
}


static void ____lambda129__property_string_changed (BoxesProperty* property, const gchar* value, gpointer self, GError** error) {
	___lambda129_ (self, property, value, error);
}


static void _g_object_unref0_ (gpointer var) {
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
}


static void _g_list_free__g_object_unref0_ (GList* self) {
	g_list_foreach (self, (GFunc) _g_object_unref0_, NULL);
	g_list_free (self);
}


static GList* boxes_remote_machine_real_get_properties (BoxesIPropertiesProvider* base, BoxesPropertiesPage page) {
	BoxesRemoteMachine * self;
	GList* result = NULL;
	GList* list;
	BoxesPropertiesPage _tmp0_;
	BoxesDisplay* _tmp24_;
	BoxesDisplay* _tmp25_;
	BoxesPropertiesPage _tmp26_;
	GList* _tmp27_ = NULL;
	self = (BoxesRemoteMachine*) base;
	list = NULL;
	_tmp0_ = page;
	switch (_tmp0_) {
		case BOXES_PROPERTIES_PAGE_LOGIN:
		{
			const gchar* _tmp1_ = NULL;
			BoxesCollectionSource* _tmp2_;
			gchar* _tmp3_;
			gchar* _tmp4_;
			gchar* _tmp5_;
			BoxesProperty* _tmp6_ = NULL;
			BoxesProperty* _tmp7_;
			const gchar* _tmp8_ = NULL;
			BoxesCollectionSource* _tmp9_;
			gchar* _tmp10_;
			gchar* _tmp11_;
			gchar* _tmp12_;
			BoxesProperty* _tmp13_ = NULL;
			BoxesProperty* _tmp14_;
			_tmp1_ = _ ("Name");
			_tmp2_ = ((BoxesMachine*) self)->source;
			_tmp3_ = boxes_collection_source_get_name (_tmp2_);
			_tmp4_ = _tmp3_;
			_tmp5_ = _tmp4_;
			_tmp6_ = boxes_iproperties_provider_add_string_property ((BoxesIPropertiesProvider*) self, &list, _tmp1_, _tmp5_, ____lambda129__property_string_changed, self);
			_tmp7_ = _tmp6_;
			_g_object_unref0 (_tmp7_);
			_g_free0 (_tmp5_);
			_tmp8_ = _ ("URI");
			_tmp9_ = ((BoxesMachine*) self)->source;
			_tmp10_ = boxes_collection_source_get_uri (_tmp9_);
			_tmp11_ = _tmp10_;
			_tmp12_ = _tmp11_;
			_tmp13_ = boxes_iproperties_provider_add_string_property ((BoxesIPropertiesProvider*) self, &list, _tmp8_, _tmp12_, NULL, NULL);
			_tmp14_ = _tmp13_;
			_g_object_unref0 (_tmp14_);
			_g_free0 (_tmp12_);
			break;
		}
		case BOXES_PROPERTIES_PAGE_DISPLAY:
		{
			const gchar* _tmp15_ = NULL;
			BoxesCollectionSource* _tmp16_;
			gchar* _tmp17_;
			gchar* _tmp18_;
			gchar* _tmp19_;
			gchar* _tmp20_ = NULL;
			gchar* _tmp21_;
			BoxesProperty* _tmp22_ = NULL;
			BoxesProperty* _tmp23_;
			_tmp15_ = _ ("Protocol");
			_tmp16_ = ((BoxesMachine*) self)->source;
			_tmp17_ = boxes_collection_source_get_source_type (_tmp16_);
			_tmp18_ = _tmp17_;
			_tmp19_ = _tmp18_;
			_tmp20_ = g_utf8_strup (_tmp19_, (gssize) (-1));
			_tmp21_ = _tmp20_;
			_tmp22_ = boxes_iproperties_provider_add_string_property ((BoxesIPropertiesProvider*) self, &list, _tmp15_, _tmp21_, NULL, NULL);
			_tmp23_ = _tmp22_;
			_g_object_unref0 (_tmp23_);
			_g_free0 (_tmp21_);
			_g_free0 (_tmp19_);
			break;
		}
		default:
		break;
	}
	_tmp24_ = boxes_machine_get_display ((BoxesMachine*) self);
	_tmp25_ = _tmp24_;
	_tmp26_ = page;
	_tmp27_ = boxes_display_get_properties (_tmp25_, _tmp26_);
	list = g_list_concat (list, _tmp27_);
	result = list;
	return result;
}


static void boxes_remote_machine_real_delete (BoxesMachine* base, gboolean by_user) {
	BoxesRemoteMachine * self;
	gboolean _tmp0_;
	BoxesCollectionSource* _tmp1_;
	self = (BoxesRemoteMachine*) base;
	_tmp0_ = by_user;
	g_return_if_fail (_tmp0_);
	_tmp1_ = ((BoxesMachine*) self)->source;
	boxes_collection_source_delete (_tmp1_);
}


static void boxes_remote_machine_class_init (BoxesRemoteMachineClass * klass) {
	boxes_remote_machine_parent_class = g_type_class_peek_parent (klass);
	BOXES_MACHINE_CLASS (klass)->connect_display = boxes_remote_machine_real_connect_display;
	BOXES_MACHINE_CLASS (klass)->connect_display_finish = boxes_remote_machine_real_connect_display_finish;
	BOXES_MACHINE_CLASS (klass)->get_properties = boxes_remote_machine_real_get_properties;
	BOXES_MACHINE_CLASS (klass)->delete = boxes_remote_machine_real_delete;
}


static void boxes_remote_machine_boxes_iproperties_provider_interface_init (BoxesIPropertiesProviderIface * iface) {
	boxes_remote_machine_boxes_iproperties_provider_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_properties = (GList* (*)(BoxesIPropertiesProvider*, BoxesPropertiesPage)) boxes_remote_machine_real_get_properties;
}


static void boxes_remote_machine_instance_init (BoxesRemoteMachine * self) {
}


GType boxes_remote_machine_get_type (void) {
	static volatile gsize boxes_remote_machine_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_remote_machine_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (BoxesRemoteMachineClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) boxes_remote_machine_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BoxesRemoteMachine), 0, (GInstanceInitFunc) boxes_remote_machine_instance_init, NULL };
		static const GInterfaceInfo boxes_iproperties_provider_info = { (GInterfaceInitFunc) boxes_remote_machine_boxes_iproperties_provider_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType boxes_remote_machine_type_id;
		boxes_remote_machine_type_id = g_type_register_static (BOXES_TYPE_MACHINE, "BoxesRemoteMachine", &g_define_type_info, 0);
		g_type_add_interface_static (boxes_remote_machine_type_id, BOXES_TYPE_IPROPERTIES_PROVIDER, &boxes_iproperties_provider_info);
		g_once_init_leave (&boxes_remote_machine_type_id__volatile, boxes_remote_machine_type_id);
	}
	return boxes_remote_machine_type_id__volatile;
}



