/* display.c generated by valac 0.17.5.2-8558, the Vala compiler
 * generated from display.vala, do not modify */

/* This file is part of GNOME Boxes. License: LGPLv2+*/

#include <glib.h>
#include <glib-object.h>
#include <libcommon.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <stdlib.h>
#include <string.h>


#define BOXES_TYPE_IPROPERTIES_PROVIDER (boxes_iproperties_provider_get_type ())
#define BOXES_IPROPERTIES_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER, BoxesIPropertiesProvider))
#define BOXES_IS_IPROPERTIES_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER))
#define BOXES_IPROPERTIES_PROVIDER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER, BoxesIPropertiesProviderIface))

typedef struct _BoxesIPropertiesProvider BoxesIPropertiesProvider;
typedef struct _BoxesIPropertiesProviderIface BoxesIPropertiesProviderIface;

#define BOXES_TYPE_PROPERTIES_PAGE (boxes_properties_page_get_type ())

#define BOXES_TYPE_PROPERTY (boxes_property_get_type ())
#define BOXES_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_PROPERTY, BoxesProperty))
#define BOXES_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_PROPERTY, BoxesPropertyClass))
#define BOXES_IS_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_PROPERTY))
#define BOXES_IS_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_PROPERTY))
#define BOXES_PROPERTY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_PROPERTY, BoxesPropertyClass))

typedef struct _BoxesProperty BoxesProperty;
typedef struct _BoxesPropertyClass BoxesPropertyClass;

#define BOXES_TYPE_DISPLAY (boxes_display_get_type ())
#define BOXES_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_DISPLAY, BoxesDisplay))
#define BOXES_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_DISPLAY, BoxesDisplayClass))
#define BOXES_IS_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_DISPLAY))
#define BOXES_IS_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_DISPLAY))
#define BOXES_DISPLAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_DISPLAY, BoxesDisplayClass))

typedef struct _BoxesDisplay BoxesDisplay;
typedef struct _BoxesDisplayClass BoxesDisplayClass;
typedef struct _BoxesDisplayPrivate BoxesDisplayPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_hash_table_unref0(var) ((var == NULL) ? NULL : (var = (g_hash_table_unref (var), NULL)))

typedef enum  {
	BOXES_PROPERTIES_PAGE_LOGIN,
	BOXES_PROPERTIES_PAGE_SYSTEM,
	BOXES_PROPERTIES_PAGE_DISPLAY,
	BOXES_PROPERTIES_PAGE_DEVICES,
	BOXES_PROPERTIES_PAGE_LAST
} BoxesPropertiesPage;

struct _BoxesIPropertiesProviderIface {
	GTypeInterface parent_iface;
	GList* (*get_properties) (BoxesIPropertiesProvider* self, BoxesPropertiesPage page);
};

struct _BoxesDisplay {
	BoxesDisplayProperties parent_instance;
	BoxesDisplayPrivate * priv;
	GHashTable* displays;
};

struct _BoxesDisplayClass {
	BoxesDisplayPropertiesClass parent_class;
	GtkWidget* (*get_display) (BoxesDisplay* self, gint n, GError** error);
	GdkPixbuf* (*get_pixbuf) (BoxesDisplay* self, gint n, GError** error);
	void (*set_enable_inputs) (BoxesDisplay* self, GtkWidget* widget, gboolean enable);
	void (*connect_it) (BoxesDisplay* self);
	void (*disconnect_it) (BoxesDisplay* self);
	GList* (*get_properties) (BoxesDisplay* self, BoxesPropertiesPage page);
	const gchar* (*get_protocol) (BoxesDisplay* self);
	gchar* (*get_uri) (BoxesDisplay* self);
};

struct _BoxesDisplayPrivate {
	gboolean _can_grab_mouse;
	gboolean _mouse_grabbed;
	gboolean _need_password;
	gboolean _need_username;
	gchar* _password;
	gchar* _username;
};


static gpointer boxes_display_parent_class = NULL;
static BoxesIPropertiesProviderIface* boxes_display_boxes_iproperties_provider_parent_iface = NULL;

GType boxes_properties_page_get_type (void) G_GNUC_CONST;
GType boxes_property_get_type (void) G_GNUC_CONST;
GType boxes_iproperties_provider_get_type (void) G_GNUC_CONST;
GType boxes_display_get_type (void) G_GNUC_CONST;
#define BOXES_DISPLAY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), BOXES_TYPE_DISPLAY, BoxesDisplayPrivate))
enum  {
	BOXES_DISPLAY_DUMMY_PROPERTY,
	BOXES_DISPLAY_PROTOCOL,
	BOXES_DISPLAY_URI,
	BOXES_DISPLAY_CAN_GRAB_MOUSE,
	BOXES_DISPLAY_MOUSE_GRABBED,
	BOXES_DISPLAY_NEED_PASSWORD,
	BOXES_DISPLAY_NEED_USERNAME,
	BOXES_DISPLAY_PASSWORD,
	BOXES_DISPLAY_USERNAME
};
GtkWidget* boxes_display_get_display (BoxesDisplay* self, gint n, GError** error);
static GtkWidget* boxes_display_real_get_display (BoxesDisplay* self, gint n, GError** error);
GdkPixbuf* boxes_display_get_pixbuf (BoxesDisplay* self, gint n, GError** error);
static GdkPixbuf* boxes_display_real_get_pixbuf (BoxesDisplay* self, gint n, GError** error);
void boxes_display_set_enable_inputs (BoxesDisplay* self, GtkWidget* widget, gboolean enable);
static void boxes_display_real_set_enable_inputs (BoxesDisplay* self, GtkWidget* widget, gboolean enable);
void boxes_display_connect_it (BoxesDisplay* self);
static void boxes_display_real_connect_it (BoxesDisplay* self);
void boxes_display_disconnect_it (BoxesDisplay* self);
static void boxes_display_real_disconnect_it (BoxesDisplay* self);
GList* boxes_display_get_properties (BoxesDisplay* self, BoxesPropertiesPage page);
static GList* boxes_display_real_get_properties (BoxesDisplay* self, BoxesPropertiesPage page);
BoxesDisplay* boxes_display_construct (GType object_type);
const gchar* boxes_display_get_protocol (BoxesDisplay* self);
gchar* boxes_display_get_uri (BoxesDisplay* self);
gboolean boxes_display_get_can_grab_mouse (BoxesDisplay* self);
void boxes_display_set_can_grab_mouse (BoxesDisplay* self, gboolean value);
gboolean boxes_display_get_mouse_grabbed (BoxesDisplay* self);
void boxes_display_set_mouse_grabbed (BoxesDisplay* self, gboolean value);
gboolean boxes_display_get_need_password (BoxesDisplay* self);
void boxes_display_set_need_password (BoxesDisplay* self, gboolean value);
gboolean boxes_display_get_need_username (BoxesDisplay* self);
void boxes_display_set_need_username (BoxesDisplay* self, gboolean value);
const gchar* boxes_display_get_password (BoxesDisplay* self);
void boxes_display_set_password (BoxesDisplay* self, const gchar* value);
const gchar* boxes_display_get_username (BoxesDisplay* self);
void boxes_display_set_username (BoxesDisplay* self, const gchar* value);
static GObject * boxes_display_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void _g_object_unref0_ (gpointer var);
static void boxes_display_finalize (GObject* obj);
static void _vala_boxes_display_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_boxes_display_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


static GtkWidget* boxes_display_real_get_display (BoxesDisplay* self, gint n, GError** error) {
	g_critical ("Type `%s' does not implement abstract method `boxes_display_get_display'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


GtkWidget* boxes_display_get_display (BoxesDisplay* self, gint n, GError** error) {
	g_return_val_if_fail (self != NULL, NULL);
	return BOXES_DISPLAY_GET_CLASS (self)->get_display (self, n, error);
}


static GdkPixbuf* boxes_display_real_get_pixbuf (BoxesDisplay* self, gint n, GError** error) {
	g_critical ("Type `%s' does not implement abstract method `boxes_display_get_pixbuf'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


GdkPixbuf* boxes_display_get_pixbuf (BoxesDisplay* self, gint n, GError** error) {
	g_return_val_if_fail (self != NULL, NULL);
	return BOXES_DISPLAY_GET_CLASS (self)->get_pixbuf (self, n, error);
}


static void boxes_display_real_set_enable_inputs (BoxesDisplay* self, GtkWidget* widget, gboolean enable) {
	g_critical ("Type `%s' does not implement abstract method `boxes_display_set_enable_inputs'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


void boxes_display_set_enable_inputs (BoxesDisplay* self, GtkWidget* widget, gboolean enable) {
	g_return_if_fail (self != NULL);
	BOXES_DISPLAY_GET_CLASS (self)->set_enable_inputs (self, widget, enable);
}


static void boxes_display_real_connect_it (BoxesDisplay* self) {
	g_critical ("Type `%s' does not implement abstract method `boxes_display_connect_it'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


void boxes_display_connect_it (BoxesDisplay* self) {
	g_return_if_fail (self != NULL);
	BOXES_DISPLAY_GET_CLASS (self)->connect_it (self);
}


static void boxes_display_real_disconnect_it (BoxesDisplay* self) {
	g_critical ("Type `%s' does not implement abstract method `boxes_display_disconnect_it'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


void boxes_display_disconnect_it (BoxesDisplay* self) {
	g_return_if_fail (self != NULL);
	BOXES_DISPLAY_GET_CLASS (self)->disconnect_it (self);
}


static GList* boxes_display_real_get_properties (BoxesDisplay* self, BoxesPropertiesPage page) {
	g_critical ("Type `%s' does not implement abstract method `boxes_display_get_properties'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


GList* boxes_display_get_properties (BoxesDisplay* self, BoxesPropertiesPage page) {
	g_return_val_if_fail (self != NULL, NULL);
	return BOXES_DISPLAY_GET_CLASS (self)->get_properties (self, page);
}


BoxesDisplay* boxes_display_construct (GType object_type) {
	BoxesDisplay * self = NULL;
	self = (BoxesDisplay*) boxes_display_properties_construct (object_type);
	return self;
}


const gchar* boxes_display_get_protocol (BoxesDisplay* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return BOXES_DISPLAY_GET_CLASS (self)->get_protocol (self);
}


gchar* boxes_display_get_uri (BoxesDisplay* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return BOXES_DISPLAY_GET_CLASS (self)->get_uri (self);
}


gboolean boxes_display_get_can_grab_mouse (BoxesDisplay* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->priv->_can_grab_mouse;
	result = _tmp0_;
	return result;
}


void boxes_display_set_can_grab_mouse (BoxesDisplay* self, gboolean value) {
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	self->priv->_can_grab_mouse = _tmp0_;
	g_object_notify ((GObject *) self, "can-grab-mouse");
}


gboolean boxes_display_get_mouse_grabbed (BoxesDisplay* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->priv->_mouse_grabbed;
	result = _tmp0_;
	return result;
}


void boxes_display_set_mouse_grabbed (BoxesDisplay* self, gboolean value) {
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	self->priv->_mouse_grabbed = _tmp0_;
	g_object_notify ((GObject *) self, "mouse-grabbed");
}


gboolean boxes_display_get_need_password (BoxesDisplay* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->priv->_need_password;
	result = _tmp0_;
	return result;
}


void boxes_display_set_need_password (BoxesDisplay* self, gboolean value) {
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	self->priv->_need_password = _tmp0_;
	g_object_notify ((GObject *) self, "need-password");
}


gboolean boxes_display_get_need_username (BoxesDisplay* self) {
	gboolean result;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->priv->_need_username;
	result = _tmp0_;
	return result;
}


void boxes_display_set_need_username (BoxesDisplay* self, gboolean value) {
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	self->priv->_need_username = _tmp0_;
	g_object_notify ((GObject *) self, "need-username");
}


const gchar* boxes_display_get_password (BoxesDisplay* self) {
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_password;
	result = _tmp0_;
	return result;
}


void boxes_display_set_password (BoxesDisplay* self, const gchar* value) {
	const gchar* _tmp0_;
	gchar* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	_tmp1_ = g_strdup (_tmp0_);
	_g_free0 (self->priv->_password);
	self->priv->_password = _tmp1_;
	g_object_notify ((GObject *) self, "password");
}


const gchar* boxes_display_get_username (BoxesDisplay* self) {
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_username;
	result = _tmp0_;
	return result;
}


void boxes_display_set_username (BoxesDisplay* self, const gchar* value) {
	const gchar* _tmp0_;
	gchar* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	_tmp1_ = g_strdup (_tmp0_);
	_g_free0 (self->priv->_username);
	self->priv->_username = _tmp1_;
	g_object_notify ((GObject *) self, "username");
}


static void _g_object_unref0_ (gpointer var) {
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
}


static GObject * boxes_display_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	BoxesDisplay * self;
	GHashFunc _tmp0_;
	GEqualFunc _tmp1_;
	GHashTable* _tmp2_;
	parent_class = G_OBJECT_CLASS (boxes_display_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BOXES_TYPE_DISPLAY, BoxesDisplay);
	_tmp0_ = g_direct_hash;
	_tmp1_ = g_direct_equal;
	_tmp2_ = g_hash_table_new_full (_tmp0_, _tmp1_, NULL, _g_object_unref0_);
	_g_hash_table_unref0 (self->displays);
	self->displays = _tmp2_;
	return obj;
}


static void boxes_display_class_init (BoxesDisplayClass * klass) {
	boxes_display_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (BoxesDisplayPrivate));
	BOXES_DISPLAY_CLASS (klass)->get_display = boxes_display_real_get_display;
	BOXES_DISPLAY_CLASS (klass)->get_pixbuf = boxes_display_real_get_pixbuf;
	BOXES_DISPLAY_CLASS (klass)->set_enable_inputs = boxes_display_real_set_enable_inputs;
	BOXES_DISPLAY_CLASS (klass)->connect_it = boxes_display_real_connect_it;
	BOXES_DISPLAY_CLASS (klass)->disconnect_it = boxes_display_real_disconnect_it;
	BOXES_DISPLAY_CLASS (klass)->get_properties = boxes_display_real_get_properties;
	G_OBJECT_CLASS (klass)->get_property = _vala_boxes_display_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_boxes_display_set_property;
	G_OBJECT_CLASS (klass)->constructor = boxes_display_constructor;
	G_OBJECT_CLASS (klass)->finalize = boxes_display_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_DISPLAY_PROTOCOL, g_param_spec_string ("protocol", "protocol", "protocol", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_DISPLAY_URI, g_param_spec_string ("uri", "uri", "uri", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_DISPLAY_CAN_GRAB_MOUSE, g_param_spec_boolean ("can-grab-mouse", "can-grab-mouse", "can-grab-mouse", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_DISPLAY_MOUSE_GRABBED, g_param_spec_boolean ("mouse-grabbed", "mouse-grabbed", "mouse-grabbed", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_DISPLAY_NEED_PASSWORD, g_param_spec_boolean ("need-password", "need-password", "need-password", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_DISPLAY_NEED_USERNAME, g_param_spec_boolean ("need-username", "need-username", "need-username", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_DISPLAY_PASSWORD, g_param_spec_string ("password", "password", "password", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_DISPLAY_USERNAME, g_param_spec_string ("username", "username", "username", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_signal_new ("show", BOXES_TYPE_DISPLAY, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
	g_signal_new ("hide", BOXES_TYPE_DISPLAY, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
	g_signal_new ("disconnected", BOXES_TYPE_DISPLAY, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void boxes_display_boxes_iproperties_provider_interface_init (BoxesIPropertiesProviderIface * iface) {
	boxes_display_boxes_iproperties_provider_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_properties = (GList* (*)(BoxesIPropertiesProvider*, BoxesPropertiesPage)) boxes_display_get_properties;
}


static void boxes_display_instance_init (BoxesDisplay * self) {
	self->priv = BOXES_DISPLAY_GET_PRIVATE (self);
}


static void boxes_display_finalize (GObject* obj) {
	BoxesDisplay * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BOXES_TYPE_DISPLAY, BoxesDisplay);
	_g_free0 (self->priv->_password);
	_g_free0 (self->priv->_username);
	_g_hash_table_unref0 (self->displays);
	G_OBJECT_CLASS (boxes_display_parent_class)->finalize (obj);
}


GType boxes_display_get_type (void) {
	static volatile gsize boxes_display_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_display_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (BoxesDisplayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) boxes_display_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BoxesDisplay), 0, (GInstanceInitFunc) boxes_display_instance_init, NULL };
		static const GInterfaceInfo boxes_iproperties_provider_info = { (GInterfaceInitFunc) boxes_display_boxes_iproperties_provider_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType boxes_display_type_id;
		boxes_display_type_id = g_type_register_static (BOXES_TYPE_DISPLAY_PROPERTIES, "BoxesDisplay", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_type_add_interface_static (boxes_display_type_id, BOXES_TYPE_IPROPERTIES_PROVIDER, &boxes_iproperties_provider_info);
		g_once_init_leave (&boxes_display_type_id__volatile, boxes_display_type_id);
	}
	return boxes_display_type_id__volatile;
}


static void _vala_boxes_display_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	BoxesDisplay * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_DISPLAY, BoxesDisplay);
	switch (property_id) {
		case BOXES_DISPLAY_CAN_GRAB_MOUSE:
		g_value_set_boolean (value, boxes_display_get_can_grab_mouse (self));
		break;
		case BOXES_DISPLAY_MOUSE_GRABBED:
		g_value_set_boolean (value, boxes_display_get_mouse_grabbed (self));
		break;
		case BOXES_DISPLAY_NEED_PASSWORD:
		g_value_set_boolean (value, boxes_display_get_need_password (self));
		break;
		case BOXES_DISPLAY_NEED_USERNAME:
		g_value_set_boolean (value, boxes_display_get_need_username (self));
		break;
		case BOXES_DISPLAY_PASSWORD:
		g_value_set_string (value, boxes_display_get_password (self));
		break;
		case BOXES_DISPLAY_USERNAME:
		g_value_set_string (value, boxes_display_get_username (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_boxes_display_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	BoxesDisplay * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_DISPLAY, BoxesDisplay);
	switch (property_id) {
		case BOXES_DISPLAY_CAN_GRAB_MOUSE:
		boxes_display_set_can_grab_mouse (self, g_value_get_boolean (value));
		break;
		case BOXES_DISPLAY_MOUSE_GRABBED:
		boxes_display_set_mouse_grabbed (self, g_value_get_boolean (value));
		break;
		case BOXES_DISPLAY_NEED_PASSWORD:
		boxes_display_set_need_password (self, g_value_get_boolean (value));
		break;
		case BOXES_DISPLAY_NEED_USERNAME:
		boxes_display_set_need_username (self, g_value_get_boolean (value));
		break;
		case BOXES_DISPLAY_PASSWORD:
		boxes_display_set_password (self, g_value_get_string (value));
		break;
		case BOXES_DISPLAY_USERNAME:
		boxes_display_set_username (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}



